@@ -82,80 +82,80 @@ int x11_native_format,have_dga=1,debug;
 
 struct event_entry kbd_events[] = {
     {
-	event:  "kbd-key-+",
-	action: "volume inc",
+	.event  = "kbd-key-+",
+	.action = "volume inc",
     },{
-	event:  "kbd-key--",
-	action: "volume dec",
+	.event  = "kbd-key--",
+	.action = "volume dec",
     },{
-	event:  "kbd-key-enter",
-	action: "volume mute",
+	.event  = "kbd-key-enter",
+	.action = "volume mute",
     },{
-	event:  "kbd-key-space",
-	action: "setstation next",
+	.event  = "kbd-key-space",
+	.action = "setstation next",
     },{
-	event:  "kbd-key-backspace",
-	action: "setstation back",
+	.event  = "kbd-key-backspace",
+	.action = "setstation back",
     },{
-	event:  "kbd-key-pgup",
-	action: "setstation prev",
+	.event  = "kbd-key-pgup",
+	.action = "setstation next",
     },{
-	event:  "kbd-key-pgdown",
-	action: "setstation next",
+	.event  = "kbd-key-pgdown",
+	.action = "setstation prev",
     },{
-	event:  "kbd-key-right",
-	action: "setchannel fine_up",
+	.event  = "kbd-key-right",
+	.action = "setchannel fine_up",
     },{
-	event:  "kbd-key-left",
-	action: "setchannel fine_down",
+	.event  = "kbd-key-left",
+	.action = "setchannel fine_down",
     },{
-	event:  "kbd-key-up",
-	action: "setchannel next",
+	.event  = "kbd-key-up",
+	.action = "setchannel next",
     },{
-	event:  "kbd-key-down",
-	action: "setchannel prev",
+	.event  = "kbd-key-down",
+	.action = "setchannel prev",
     },{
-	event:  "kbd-key-g",
-	action: "snap ppm",
+	.event  = "kbd-key-g",
+	.action = "snap ppm",
     },{
-	event:  "kbd-key-j",
-	action: "snap jpeg",
+	.event  = "kbd-key-j",
+	.action = "snap jpeg",
     },{
-	event:  "kbd-key-v",
-	action: "capture toggle",
+	.event  = "kbd-key-v",
+	.action = "capture toggle",
     },{
-	event:  "kbd-key-f",
-	action: "fullscreen toggle",
+	.event  = "kbd-key-f",
+	.action = "fullscreen toggle",
     },{
-	event:  "kbd-key-0",
-	action: "keypad 0",
+	.event  = "kbd-key-0",
+	.action = "keypad 0",
     },{
-	event:  "kbd-key-1",
-	action: "keypad 1",
+	.event  = "kbd-key-1",
+	.action = "keypad 1",
     },{
-	event:  "kbd-key-2",
-	action: "keypad 2",
+	.event  = "kbd-key-2",
+	.action = "keypad 2",
     },{
-	event:  "kbd-key-3",
-	action: "keypad 3",
+	.event  = "kbd-key-3",
+	.action = "keypad 3",
     },{
-	event:  "kbd-key-4",
-	action: "keypad 4",
+	.event  = "kbd-key-4",
+	.action = "keypad 4",
     },{
-	event:  "kbd-key-5",
-	action: "keypad 5",
+	.event  = "kbd-key-5",
+	.action = "keypad 5",
     },{
-	event:  "kbd-key-6",
-	action: "keypad 6",
+	.event  = "kbd-key-6",
+	.action = "keypad 6",
     },{
-	event:  "kbd-key-7",
-	action: "keypad 7",
+	.event  = "kbd-key-7",
+	.action = "keypad 7",
     },{
-	event:  "kbd-key-8",
-	action: "keypad 8",
+	.event  = "kbd-key-8",
+	.action = "keypad 8",
     },{
-	event:  "kbd-key-9",
-	action: "keypad 9",
+	.event  = "kbd-key-9",
+	.action = "keypad 9",
     },{
 
 	/* end of list */
@@ -187,7 +187,6 @@ static struct KEYTAB keytab[] = {
 
 #define NKEYTAB (sizeof(keytab)/sizeof(struct KEYTAB))
 
-static char              *snapbase;
 static char              default_title[128] = "???";
 static char              message[128] = "";
 
@@ -195,12 +194,23 @@ static char              message[128] = "";
 /* framebuffer stuff                                                      */
 
 static void
-linear_palette(int bit)
+linear_palette(int r, int g, int b)
 {
-    int             i, size = 256 >> (8 - bit);
+    int i, size;
 
+    size = 256 >> (8 - r);
     for (i = 0; i < size; i++)
-	red[i] = green[i] = blue[i] = (unsigned short)(65535.0
+	red[i] = (unsigned short)(65535.0
+		* pow(i/(size - 1.0), fbgamma));
+
+    size = 256 >> (8 - g);
+    for (i = 0; i < size; i++)
+	green[i] = (unsigned short)(65535.0
+		* pow(i/(size - 1.0), fbgamma));
+
+    size = 256 >> (8 - b);
+    for (i = 0; i < size; i++)
+	blue[i] = (unsigned short)(65535.0
 		* pow(i/(size - 1.0), fbgamma));
 }
 
@@ -232,7 +242,7 @@ fb_initcolors(int fd, int gray)
     switch (fb_var.bits_per_pixel) {
     case 8:
 	if (gray) {
-	    linear_palette(8);
+	    linear_palette(8,8,8);
 	    x11_native_format = VIDEO_GRAY;
 	} else {
 	    dither_palette(5,9,5);
@@ -242,7 +252,9 @@ fb_initcolors(int fd, int gray)
     case 15:
     case 16:
 	if (fb_fix.visual == FB_VISUAL_DIRECTCOLOR)
-	    linear_palette(5);
+	    linear_palette(fb_var.red.length,
+			   fb_var.green.length,
+			   fb_var.blue.length);
 #if BYTE_ORDER == BIG_ENDIAN
 	x11_native_format = (fb_var.green.length == 6) ?
 	    VIDEO_RGB16_BE : VIDEO_RGB15_BE;
@@ -253,7 +265,7 @@ fb_initcolors(int fd, int gray)
 	break;
     case 24:
 	if (fb_fix.visual == FB_VISUAL_DIRECTCOLOR)
-	    linear_palette(8);
+	    linear_palette(8,8,8);
 #if BYTE_ORDER == BIG_ENDIAN
 	x11_native_format = VIDEO_RGB24;
 #else
@@ -262,7 +274,7 @@ fb_initcolors(int fd, int gray)
 	break;
     case 32:
 	if (fb_fix.visual == FB_VISUAL_DIRECTCOLOR)
-	    linear_palette(8);
+	    linear_palette(8,8,8);
 #if BYTE_ORDER == BIG_ENDIAN
 	x11_native_format = VIDEO_RGB32;
 #else
