@@ -62,7 +62,7 @@ char  *grab_norm  = NULL;
 /* jpeg stuff                                                             */
 
 static int
-write_file(int fd, char *data, int width, int height)
+write_file(int fd, unsigned char *data, int width, int height)
 {
     struct jpeg_compress_struct cinfo;
     struct jpeg_error_mgr jerr;
@@ -421,7 +421,7 @@ add_text(char *image, int width, int height)
     time(&t);
     tm = localtime(&t);
     len = strftime(line,MSG_MAXLEN,get_message(),tm);
-    fprintf(stderr,"%s\n",line);
+    // fprintf(stderr,"%s\n",line);
 
     for (y = 0; y < CHAR_HEIGHT; y++) {
 	ptr = image + 3 * width * (height-CHAR_HEIGHT-2+y) + 12;
